@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 module Asciidoctor
 # A built-in {Converter} implementation that generates HTML 5 output
-# consistent with the html5 backend from AsciiDoc Python.
+# consistent with the html5 backend from AsciiDoc.py.
 class Converter::Html5Converter < Converter::Base
   register_for 'html5'
 
@@ -1111,14 +1111,17 @@ Your browser does not support the video tag.
       else
         attrs = node.role ? %( class="#{node.role}") : ''
         unless (text = node.text)
-          refid = node.attributes['refid']
-          if AbstractNode === (ref = (@refs ||= node.document.catalog[:refs])[refid]) && (@resolving_xref ||= (outer = true)) && outer
-            if !(text = ref.xreftext node.attr 'xrefstyle', nil, true)
-              text = %([#{refid}])
-            elsif text.include? '<a'
-              text = text.gsub DropAnchorRx, ''
+          if AbstractNode === (ref = (@refs ||= node.document.catalog[:refs])[refid = node.attributes['refid']] || (refid.nil_or_empty? ? (top = get_root_document node) : nil))
+            if (@resolving_xref ||= (outer = true)) && outer
+              if (text = ref.xreftext node.attr 'xrefstyle', nil, true)
+                text = text.gsub DropAnchorRx, '' if text.include? '<a'
+              else
+                text = top ? '[^top]' : %([#{refid}])
+              end
+              @resolving_xref = nil
+            else
+              text = top ? '[^top]' : %([#{refid}])
             end
-            @resolving_xref = nil
           else
             text = %([#{refid}])
           end
@@ -1268,17 +1271,13 @@ Your browser does not support the video tag.
       old_start_tag = new_start_tag = start_tag_match = nil
       # NOTE width, height and style attributes are removed if either width or height is specified
       ['width', 'height'].each do |dim|
-        if node.attr? dim
-          unless new_start_tag
-            next if (start_tag_match ||= (svg.match SvgStartTagRx) || :no_match) == :no_match
-            new_start_tag = (old_start_tag = start_tag_match[0]).gsub DimensionAttributeRx, ''
-          end
-          unless (dim_val = node.attr dim).end_with? '%'
-            # QUESTION should we add px since it's already the default?
-            dim_val += 'px'
-          end
-          new_start_tag = %(#{new_start_tag.chop} #{dim}="#{dim_val}">)
+        next unless node.attr? dim
+        unless new_start_tag
+          next if (start_tag_match ||= (svg.match SvgStartTagRx) || :no_match) == :no_match
+          new_start_tag = (old_start_tag = start_tag_match[0]).gsub DimensionAttributeRx, ''
         end
+        # NOTE a unitless value in HTML is assumed to be px, so we can pass the value straight through
+        new_start_tag = %(#{new_start_tag.chop} #{dim}="#{node.attr dim}">)
       end
       svg = %(#{new_start_tag}#{svg[old_start_tag.length..-1]}) if new_start_tag
     end
@@ -1314,10 +1313,17 @@ Your browser does not support the video tag.
     manname_id_attr = (manname_id = node.attr 'manname-id') ? %( id="#{manname_id}") : ''
     %(<h2#{manname_id_attr}>#{manname_title}</h2>
 <div class="sectionbody">
-<p>#{node.attr 'manname'} - #{node.attr 'manpurpose'}</p>
+<p>#{(node.attr 'mannames').join ', '} - #{node.attr 'manpurpose'}</p>
 </div>)
   end
 
+  def get_root_document node
+    while (node = node.document).nested?
+      node = node.parent_document
+    end
+    node
+  end
+
   # NOTE adapt to older converters that relied on unprefixed method names
   def method_missing id, *params
     !((name = id.to_s).start_with? 'convert_') && (handles? name) ? (send %(convert_#{name}), *params) : super
