@@ -1,6 +1,7 @@
 # frozen_string_literal: true
 # NOTE remove once minimum required Ruby version is at least 2.4
-Float.prepend(Module.new do
+# NOTE use `send :prepend` to be nice to Ruby 2.0
+Float.send :prepend, (Module.new do
   def truncate *args
     if args.length == 1
       if (precision = Integer args.shift) == 0
